#include "api.h"
#include "cos_api.h"
#include "cos_sys_config.h"
#include "cos_config.h"
#include "util/auth_tool.h"
#include "Poco/TaskManager.h"

#include <stdlib.h>
#include <sys/stat.h>
#include <iostream>
#include <map>
#include <string>
#include <thread>
#include <vector>

using namespace qcloud_cos;
CosConfig g_config;
std::string g_strBucketName = "";

void result2ufileError(ufile_error & retError, qcloud_cos::CosResult * tRet)
{
	strncpy_s(retError.errorCode, tRet->GetErrorCode().c_str(), sizeof(retError.errorCode));
	strncpy_s(retError.errorMessage, tRet->GetErrorMsg().c_str(), sizeof(retError.errorMessage));
	strncpy_s(retError.resourceAddr, tRet->GetResourceAddr().c_str(), sizeof(retError.resourceAddr));
	strncpy_s(retError.cosRequestId, tRet->GetXCosRequestId().c_str(), sizeof(retError.cosRequestId));
	strncpy_s(retError.cosTraceId, tRet->GetXCosTraceId().c_str(), sizeof(retError.cosTraceId));
	retError.httpStatus = tRet->GetHttpStatus();
	retError.bIsSuccess = tRet->IsSucc();
}

ufile_error ufile_load_config_from_json(const char * json_buf, ufile_config * cfg)
{
	ufile_error retError = { false, 0, "", "", "", "", "" };
	if (nullptr == cfg)
		return retError;
	Poco::JSON::Parser parser;
	Poco::Dynamic::Var result;
	try {
		result = parser.parse(json_buf);
	}
	catch (Poco::JSON::JSONException& jsonErr) {
		strncpy_s(retError.errorMessage, jsonErr.message().c_str(), sizeof(retError.errorMessage));
		return retError;
	}
	if (result.type() != typeid(Poco::JSON::Object::Ptr)) {
		strncpy_s(retError.errorMessage, "failed to parse config file", sizeof(retError.errorMessage));
		return retError;
	}
	Poco::JSON::Object::Ptr object = result.extract<Poco::JSON::Object::Ptr>();

	uint64_t appid = 0;
	std::string strPublicKey = "";
	std::string strPrivateKey = "";
	std::string strRegion = "";
	CosConfig::JsonObjectGetIntegerValue(object, "AppID", &appid);
	CosConfig::JsonObjectGetStringValue(object, "AccessKey", &strPublicKey);
	CosConfig::JsonObjectGetStringValue(object, "SecretId", &strPublicKey);
	CosConfig::JsonObjectGetStringValue(object, "SecretKey", &strPrivateKey);
	CosConfig::JsonObjectGetStringValue(object, "Region", &strRegion);
	cfg->appid = appid;
	cfg->public_key = strPublicKey.c_str();
	cfg->private_key = strPrivateKey.c_str();
	cfg->region = strRegion.c_str();
	retError.bIsSuccess = true;
	return retError;
}

void ufile_free_config(ufile_config cfg)
{
	//˺޵ãҲʵ
}

ufile_error ufile_sdk_initialize(const ufile_config cfg, int open_verbose)
{
	UNUSED_PARAM(open_verbose);
	ufile_error retError = { false, 0, "", "", "", "", "" };
	CosConfig config(cfg.appid, cfg.public_key, cfg.private_key, cfg.region);
	g_config = config;
	g_strBucketName = cfg.bucket_host;
	retError.bIsSuccess = true;
	return retError;
}

void ufile_sdk_cleanup()
{
}

ufile_error ufile_head(const char * bucket_name, const char * key, ufile_file_info * info)
{
	//˺޵ãҲʵ
	return ufile_error();
}

void ufile_free_file_info(ufile_file_info info)
{
	//˺޵ãҲʵ
}

ufile_error ufile_put_buf(const char * bucket_name, const char * key, const char * mime_type, char * buffer, size_t buf_len)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_put_file(const char * bucket_name, const char * key, const char * mime_type, FILE * file)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_put_file_by_filepath(const char * bucket_name, const char * key, const char * mime_type, const char * file_path)
{
	UNUSED_PARAM(mime_type);
	ufile_error retError = { false, 0, "", "", "", "", "" };
	std::string strBucketName = bucket_name;
	if (strBucketName.empty())
		strBucketName = g_strBucketName;
	qcloud_cos::PutObjectByFileReq req(strBucketName, key, file_path);
	qcloud_cos::PutObjectByFileResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.PutObject(req, &resp);
	result2ufileError(retError, &result);
	return retError;
}

void set_upload_company(int company)
{
}

ufile_error ufile_delete(const char * bucket_name, const char * key)
{
	ufile_error retError = { false, 0, "", "", "", "", "" };
	std::string strBucketName = bucket_name;
	if (strBucketName.empty())
		strBucketName = g_strBucketName;
	qcloud_cos::DeleteObjectReq req(strBucketName, key);
	qcloud_cos::DeleteObjectResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.DeleteObject(req, &resp);
	result2ufileError(retError, &result);
	return ufile_error();
}

ufile_error ufile_multiple_upload_init(ufile_mutipart_state * self, const char * bucket_name, const char * key, const char * mime_type)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_multiple_upload_part(ufile_mutipart_state * self, char * buffer, size_t buf_len, int part_number)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_multiple_upload_finish(ufile_mutipart_state * self)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_multiple_upload_abort(ufile_mutipart_state * self)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_download(const char * bucket_name, const char * key, FILE * file, size_t * return_size, const char * path)
{
	ufile_error retError = { false, 0, "", "", "", "", "" };
	if (nullptr == file)
	{
		strncpy_s(retError.errorMessage, "file is nullptr", sizeof(retError.errorMessage));
		return retError;
	}
	std::ostringstream os;
	std::string strBucketName = bucket_name;
	if (strBucketName.empty())
		strBucketName = g_strBucketName;
	qcloud_cos::GetObjectByStreamReq req(strBucketName, key, os);
	qcloud_cos::GetObjectByStreamResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.GetObject(req, &resp);
	if (result.IsSucc())
	{
		const char * tBuf = os.str().c_str();
		fwrite(tBuf, sizeof(tBuf), strlen(tBuf), file);
	}
	result2ufileError(retError, &result);
	return retError;
}

ufile_error ufile_download_piece(const char * bucket_name, const char * key, size_t start_position, char * buf, size_t buf_len, size_t * return_size)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_bucket_create(const char * bucket_name, const char * region, const char * bucket_type)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_bucket_delete(const char * bucket_name)
{
	ufile_error retError = { false, 0, "", "", "", "", "" };
	std::string strBucketName = bucket_name;
	if (strBucketName.empty())
		strBucketName = g_strBucketName;
	qcloud_cos::DeleteBucketReq req(strBucketName);
	qcloud_cos::DeleteBucketResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.DeleteBucket(req, &resp);
	result2ufileError(retError, &result);
	return ufile_error();
}
